/*
 Copyright (c) 2013 yvt
 
 This file is part of OpenSpades.
 
 OpenSpades is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
 
 OpenSpades is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with OpenSpades.  If not, see <http://www.gnu.org/licenses/>.
 
 */
 namespace spades 
 {
	class ViewShotgunSkin: 
	IToolSkin, IViewToolSkin, IWeaponSkin,
	BasicViewWeapon 
	{
		
		private AudioDevice@ audioDevice;
		private Model@ gunModel;

		private Model@ pumpModel;

		private Model@ sightModel2;
		
		private Image@ reddot;

		
		private AudioChunk@ fireSound;
		private AudioChunk@ fireemptySound;
		private AudioChunk@ reloadSound;
		private AudioChunk@ cockSound;
		
		private ConfigItem@ cg_fov = ConfigItem("cg_fov");
		
		int bullet; 		//number of bullets in current magazine
		int reloadtype;		//0 for empty; 1 for tactical
		int rbullet;
		int rtype;
		int basebullet;
		bool first = true;		
		
		// pivot of the weapon when viewed in slab6
		private Vector3 pivot = Vector3(3.50, -10.0, 20.0);
		// scale of the weapon
		private float globalScale = 0.01;
		// scale of the magazine, relative to the global scale
		private float magazineScale = 0.5;
		// delta time between each frame
		private float deltatime = 0.0;
		// checks if mag has been thrown
		// reset to false whenever reloading
		private bool hasThrownMag = false;
		
		// Creates a rotation matrix from euler angles (in the form of a Vector3) x-y-z
		Matrix4 CreateEulerAnglesMatrix( Vector3 angles ) {
			Matrix4 mat = CreateRotateMatrix( Vector3(1.0, 0.0, 0.0), angles.x );
			mat = CreateRotateMatrix( Vector3(0.0, 1.0, 0.0), angles.y ) * mat;
			mat = CreateRotateMatrix( Vector3(0.0, 0.0, 1.0), angles.z ) * mat;
			
			return mat;
		}
		// select easing functions
		float quadraticIn(float per) {
			return (per*per);
		}
		
		float quadraticOut(float per) {
			return -(per * (per-2));
		}
		
		float quadraticInOut(float per) {
			per < per/2; 
			return (per*per);
			
			per > per/2; 
			return -(per * (per-2));
			
		}	
		
		float cubicIn(float per) {
			return (per*per*per);
		}
		
		float cubicOut(float per) {
			per = per-1;
			return (per*per*per + 1);
		}
		
		ViewShotgunSkin(Renderer@ r, AudioDevice@ dev)
		{
			super(r);
			@audioDevice = dev;
			@gunModel = renderer.RegisterModel
				("Models/Weapons/Shotgun/WeaponNoPump.kv6");

			@pumpModel = renderer.RegisterModel
				("Models/Weapons/Shotgun/Pump.kv6");

			@sightModel2 = renderer.RegisterModel
				("Models/Weapons/Shotgun/Sight2.kv6");
				
			@reddot = renderer.RegisterImage
				("Gfx/DotSight.png"); 

				
			@fireSound = dev.RegisterSound
				("Sounds/Weapons/Shotgun/Fire.wav");
			@fireemptySound = dev.RegisterSound
				("Sounds/Weapons/Shotgun/FireEmpty.wav");
			@reloadSound = dev.RegisterSound
				("Sounds/Weapons/Shotgun/Reload.wav");
			@cockSound = dev.RegisterSound
				("Sounds/Weapons/Shotgun/Cock.wav");
		}
		
		void Update(float dt) 
		{
			BasicViewWeapon::Update(dt);
		}
		
		void WeaponFired()
		{
			BasicViewWeapon::WeaponFired();
			
			bullet -= 1; //decrease after each shot
			if (bullet < 1) //just make sure it's not negative
			{
				reloadtype = 0;
				bullet = 0;
			}
			basebullet = bullet;
			if(!IsMuted && bullet != 0)
			{
				Vector3 origin = Vector3(0.f, -0.3f, 0.5f);
				AudioParam param;
				param.volume = 0.3f;
				audioDevice.PlayLocal(fireSound, origin, param);
			}
			if(!IsMuted && bullet == 0)
			{
				Vector3 origin = Vector3(0.f, -0.3f, 0.5f);
				AudioParam param;
				param.volume = 0.3f;
				audioDevice.PlayLocal(fireemptySound, origin, param);
			}
			
		}
		
		void ReloadingWeapon() 
		{
			if (bullet < 1)
			{
				reloadtype = 0;
				bullet = 0;
			}
			bullet += 1; //increase after inserted shell
			if(!IsMuted)
			{
				Vector3 origin = Vector3(0.f, -0.3f, 0.5f);
				AudioParam param;
				param.volume = 0.2f;
				audioDevice.PlayLocal(reloadSound, origin, param);
			}
		}
		
		void ReloadedWeapon() 
		{
			if(!IsMuted && reloadtype == 0)
			{
				Vector3 origin = Vector3(0.f, -0.3f, 0.5f);
				AudioParam param;
				param.volume = 0.2f;
				audioDevice.PlayLocal(cockSound, origin, param);
			}
			// reset everything
			basebullet = ClipSize;
			bullet = ClipSize;
			rbullet = 0;
		}
		
		float GetZPos() 
		{
			return 0.2f - AimDownSightStateSmooth * 0.0535f;
		}
		
		// rotates gun matrix to ensure the sight is in
		// the center of screen (0, ?, 0).
		Matrix4 AdjustToAlignSight(Matrix4 mat, Vector3 sightPos, float fade) 
		{
			Vector3 p = mat * sightPos;
			mat = CreateRotateMatrix(Vector3(0.f, 0.f, 1.f), atan(p.x / p.y) * fade) * mat;
			mat = CreateRotateMatrix(Vector3(-1.f, 0.f, 0.f), atan(p.z / p.y) * fade) * mat;
			//mat = CreateRotateMatrix(Vector3(0.f, 1.f, 0.f), atan(p.z / p.x) * fade) * mat;//Optional gangsta style ADS WARNING, NOT COMPLETE
			return mat;
		}
		
		void Draw2D() {
		if(AimDownSightState > 0.6) {
				
				/*if (0.55 > ReadyState || ReadyState < 0.9) {
				//draw nothing
				}
				else {*/
				// draw reddot image
				float reddotOpacity = (AimDownSightState > 0.8f) ? AimDownSightState * 5.f - 4.f : 0.f;
				renderer.ColorP = Vector4(reddotOpacity, reddotOpacity, reddotOpacity, 0.0f); // premultiplied alpha
				
				float sightSize = 0.019f;
				
				// scale sight according to the fov value
				float fov = cg_fov.FloatValue;
				fov = tan(fov * 0.5f * 3.141592654f / 180.f);
				sightSize /= fov;
				sightSize *= renderer.ScreenHeight;
				
				// scale sight according to the distance from the eye to the sight
				Matrix4 sightMat = GetViewWeaponMatrix();
				Vector3 sightPos = sightMat * Vector3(0.f, 0.f, 0.f);
				float scale = 0.5f / sightPos.y;
				sightSize *= scale;
				
				renderer.DrawImage(reddot,
					AABB2((renderer.ScreenWidth - sightSize) * 0.5f,
							(renderer.ScreenHeight - sightSize) * 0.5f,
							sightSize, sightSize));
				return;
				//}
			}

			BasicViewWeapon::Draw2D();

		}
// redefined from BasicViewWeapon.as
		Matrix4 GetViewWeaponMatrix() {	
			Matrix4 mat;
			// sprinting animation					
			if(sprintState > 0.0) {
				sprintState = quadraticIn(sprintState);
				mat = CreateEulerAnglesMatrix(Vector3(0.2, -0.0, -0.2)*sprintState) * mat;
				mat = CreateTranslateMatrix(Vector3(0.1, -0.1, 0.05)*sprintState) * mat;
			}
			
			// raise gun animation
			if(raiseState < 1.0) {
				float putdown = 1.0 - raiseState;
				putdown = cubicIn(putdown);
				mat = CreateRotateMatrix(Vector3(0.0, 0.0, 1.0),
					putdown * -1.3) * mat;
				mat = CreateRotateMatrix(Vector3(0.0, 1.0, 0.0),
					putdown * 0.2) * mat;
				mat = CreateTranslateMatrix(Vector3(0.1, -0.3, 0.8)
					* putdown) * mat;
			}
			
			// recoil animation
			Vector3 recoilRot;
			Vector3 recoilOffset;
			if(readyState < 0.1) {
				float per = (readyState/0.1);
				per = cubicOut(per);
				recoilRot = Vector3(-0.1, 0.0, 0.0) * per;
				recoilOffset = Vector3(0.0, -0.08, -0.02) * per;
			} else if(readyState < 0.2) {
				recoilRot = Vector3(-0.1, 0.0, 0.0);
				recoilOffset = Vector3(0.0, -0.08, -0.02);
			} else if(readyState < 0.4) {
				float per = ( (readyState-0.2)/(0.4-0.2) );
				per = SmoothStep(per);
				recoilRot = Mix(Vector3(-0.1, 0.0, 0.0), Vector3(0.05, 0.0, 0.0), per);
				recoilOffset = Mix(Vector3(0.0, -0.08, -0.02), Vector3(0.0, 0.0, 0.0), per);
			} else if(readyState < 0.8) {
				float per = ( (readyState-0.4)/(0.8-0.4) );
				per = SmoothStep(per);
				recoilRot = Mix(Vector3(0.05, 0.0, 0.0), Vector3(0.0, 0.0, 0.0), per);
				recoilOffset = Vector3(0.0, 0.0, 0.0);
			}
			// No recoil when the player is aiming. Multiply by (1 - aimScopingState)
			float unSightState = 1.0-AimDownSightStateSmooth;
			mat = CreateEulerAnglesMatrix(recoilRot*unSightState) * mat;
			mat = mat * CreateTranslateMatrix(recoilOffset*unSightState);
			
			// default offset from when the player is not aiming (i.e. default position)
			mat = CreateTranslateMatrix( Mix( Vector3(-0.13, 0.5,0.2), Vector3(0.0, 0.3, -(0.-pivot.z)*globalScale), AimDownSightStateSmooth)) * mat; 

			// offset from when the player is walking
			// again, don't move the gun when the weapon is aimed
			mat = CreateTranslateMatrix(swing * GetMotionGain() * unSightState) * mat;

			// twist the gun when strafing
			// don't rotate when scoped
			mat = mat * CreateEulerAnglesMatrix(Vector3(0.0, 2.0*swing.x, 0.0)*unSightState);
			
			return mat;
		}
		
//ADD TO SCENE
		void AddToScene() 
		{
			Matrix4 mat = CreateScaleMatrix(0.020f)  * CreateTranslateMatrix(0.0, 5.0, 0.0);
			renderer.DrawImage(reddot, Vector2((renderer.ScreenWidth - reddot.Width) * 1.0f, (renderer.ScreenHeight - reddot.Height) * 1.0f));
			mat = GetViewWeaponMatrix() * mat;
			
			bool reloading = IsReloading;
			float reload = ReloadProgress*0.5f;
			Vector3 leftHand, rightHand;
			
			Vector3 leftHand2 = mat * Vector3(1.f, -9.f, 0.f); //-10
			Vector3 leftHand3 = mat * Vector3(-5.f, 2.f, 0.f); //1
			Vector3 leftHand4 = mat * Vector3(0.f, 4.f, 4.f); //1
			Vector3 leftHando = mat * Vector3(0.f, 9.f, 3.f); //1
			//CTranslateM	(x,y,z)		(x+left/-right, y+further/-closer, z+down/-up)
			
			if(AimDownSightStateSmooth > 0.8f)
			{
				mat = AdjustToAlignSight(mat, Vector3(0.f, 2.0f, -10.4f),
				(AimDownSightStateSmooth - 0.8f) / 0.2f);
			}
		
//USED FOR PUMP ACTION TYPE
if (first)
	{
		bullet = ClipSize;
		reloadtype = 1;
		first = false;
	}
if (bullet > ClipSize)
	{
		bullet = ClipSize;
	}
//END OF USED FOR PUMP ACTION
//SHELL INSERTION TYPE
rbullet = bullet - basebullet;

if (rbullet <= 1 && bullet == ClipSize)
{

}
//if this is the first and last shell
			//in this cycle everything must be HALVED
			if (rbullet <= 1 && bullet == ClipSize && reload < 0.53f) 
			{
				if(reload < 0.2f) //grab new bullet
				{
					float per = reload / 0.2f;
					per = quadraticInOut(per);
					mat = mat * CreateTranslateMatrix( Mix (Vector3(0.0, 0.0, 0.0) , Vector3(0.0, 0.0, 0.0), per ));
					mat = mat * CreateEulerAnglesMatrix( Mix (Vector3(0.0, 0.0, 0.0) , Vector3(0.0, 1.0, 0.0), per ));
					
				}
				else if(reload < 0.35f) //insert bullet
				{
					float per = (reload - 0.2f) / 0.15f;
					per = quadraticInOut(per);
					mat = mat * CreateTranslateMatrix( Mix (Vector3(0.0, 0.0, 0.0) , Vector3(0.0, 1.0, 0.0), per ));
					mat = mat * CreateEulerAnglesMatrix( Mix (Vector3(0.0, 1.0, 0.0) , Vector3(0.0, 1.0, 0.0), per ));
				}
				else if(reload < 0.53f) //move hand to original
				{
					float per = (reload - 0.35f) / 0.17f;
					per = quadraticInOut(per);
					mat = mat * CreateTranslateMatrix( Mix (Vector3(0.0, 1.0, 0.0), Vector3(0.0, 0.0, 0.0), per ));
					mat = mat * CreateEulerAnglesMatrix( Mix (Vector3(0.0, 1.0, 0.0) , Vector3(0.0, 0.0, 0.0), per ));
				}
			}
//reload animation of 1st shell
			//the weapon rotation must be slower for 1st time
			else if (rbullet <= 1 && reload < 0.52 && bullet < ClipSize) //if no shells have been inserted yet
			{
				if(reload < 0.2f) //grab new bullet
				{
					float per = reload / 0.2f;
					per = quadraticInOut(per);
					mat = mat * CreateTranslateMatrix( Mix (Vector3(0.0, 0.0, 0.0) , Vector3(-10.0, 5.0, -9.0), per ));
					mat = mat * CreateEulerAnglesMatrix( Mix (Vector3(0.0, 0.0, 0.0) , Vector3(-1.5, 0.2, -1.8), per ));
					
				}
				else if(reload < 0.35f) //insert bullet
				{
					float per = (reload - 0.2f) / 0.15f;
					per = quadraticInOut(per);
					mat = mat * CreateTranslateMatrix( Mix (Vector3(-10.0, 5.0, -9.0) , Vector3(-10.0, 5.3, -9.0), per ));
					mat = mat * CreateEulerAnglesMatrix( Mix (Vector3(-1.5, 0.2, -1.8) , Vector3(-1.3, 0.2, -1.8), per ));
				}
				else if(reload < 0.53f) //move hand to original
				{
					float per = (reload - 0.35f) / 0.17f;
					per = quadraticInOut(per);
					mat = mat * CreateTranslateMatrix( Mix (Vector3(-10.0, 5.3, -9.0) , Vector3(-10.0, 5.0, -9.0), per ));
					mat = mat * CreateEulerAnglesMatrix( Mix (Vector3(-1.3, 0.2, -1.8) , Vector3(-1.5, 0.2, -1.8), per ));
				}
			}
//reload animation of other shells
			//if shells have been inserted but its not full yet
			else if (rbullet > 1 && reload < 0.52 && bullet < ClipSize) 
			{
				if(reload < 0.2f) //grab new bullet
				{
					float per = reload / 0.2f;
					per = quadraticInOut(per);
					mat = mat * CreateTranslateMatrix( Mix (Vector3(-10.0, 5.0, -9.0) , Vector3(-10.0, 5.3, -9.0), per ));
					mat = mat * CreateEulerAnglesMatrix( Mix (Vector3(-1.5, 0.2, -1.8) , Vector3(-1.3, 0.2, -1.8), per ));
					
				}
				else if(reload < 0.35f) //insert bullet
				{
					float per = (reload - 0.2f) / 0.15f;
					per = quadraticInOut(per);
					mat = mat * CreateTranslateMatrix( Mix (Vector3(-10.0, 5.3, -9.0) , Vector3(-10.0, 5.0, -9.0), per ));
					mat = mat * CreateEulerAnglesMatrix( Mix (Vector3(-1.3, 0.2, -1.8) , Vector3(-1.5, 0.2, -1.8), per ));
				}
				else if(reload < 0.53f) //move hand to original
				{
					float per = (reload - 0.35f) / 0.17f;
					per = quadraticInOut(per);
					mat = mat * CreateTranslateMatrix( Mix (Vector3(-10.0, 5.0, -9.0) , Vector3(-10.0, 5.0, -9.0), per ));
					mat = mat * CreateEulerAnglesMatrix( Mix (Vector3(-1.5, 0.2, -1.8) , Vector3(-1.5, 0.2, -1.8), per ));
				}
			}
//reload animation of last shell
	//if reloading weapon with bullets in
			else if((rbullet + basebullet == ClipSize) && reload < 0.53 && bullet == ClipSize && reloadtype == 1) 
			{
				if(reload < 0.2f) //grab new bullet
				{
				float per = reload / 0.2f;
					per = quadraticInOut(per);
					mat = mat * CreateTranslateMatrix( Mix (Vector3(-10.0, 5.0, -9.0) , Vector3(-10.0, 5.3, -9.0), per ));
					mat = mat * CreateEulerAnglesMatrix( Mix (Vector3(-1.5, 0.2, -1.8) , Vector3(-1.3, 0.2, -1.8), per ));
				}
				else if(reload < 0.35f) //insert bullet
				{
					float per = (reload - 0.2f) / 0.15f;
					per = quadraticInOut(per);
					mat = mat * CreateTranslateMatrix( Mix (Vector3(-10.0, 5.3, -9.0) , Vector3(-10.0, 5.3, -9.0), per ));
					mat = mat * CreateEulerAnglesMatrix( Mix (Vector3(-1.3, 0.2, -1.8) , Vector3(-1.5, 0.2, -1.8), per ));
				}
				else if(reload <= 0.52f) //move hand to original
				{
					float per = (reload - 0.35f) / 0.17f;
					per = quadraticInOut(per);
					mat = mat * CreateTranslateMatrix( Mix (Vector3(-10.0, 5.3, -9.0) , Vector3(0.0, 0.0, 0.0), per ));
					mat = mat * CreateEulerAnglesMatrix( Mix (Vector3(-1.5, 0.2, -1.8) , Vector3(0.0, 0.0, 0.0), per ));
				}
			}
			//if reloading dry weapon
			else if((rbullet + basebullet == ClipSize) && reload < 0.53 && bullet == ClipSize && reloadtype == 0) 
			{
				if(reload < 0.2f) //grab new bullet
				{
				float per = reload / 0.2f;
					per = quadraticInOut(per);
					mat = mat * CreateTranslateMatrix( Mix (Vector3(-10.0, 5.0, -9.0) , Vector3(-10.0, 5.3, -9.0), per ));
					mat = mat * CreateEulerAnglesMatrix( Mix (Vector3(-1.5, 0.2, -1.8) , Vector3(-1.3, 0.2, -1.8), per ));
				}
				else if(reload < 0.35f) //insert bullet
				{
					float per = (reload - 0.2f) / 0.15f;
					per = quadraticInOut(per);
					mat = mat * CreateTranslateMatrix( Mix (Vector3(-10.0, 5.3, -9.0) , Vector3(-10.0, 5.3, -9.0), per ));
					mat = mat * CreateEulerAnglesMatrix( Mix (Vector3(-1.3, 0.2, -1.8) , Vector3(-1.5, 0.2, -1.8), per ));
				}
				else if(reload <= 0.52f) //move hand to original
				{
					float per = (reload - 0.35f) / 0.17f;
					per = quadraticInOut(per);
					mat = mat * CreateTranslateMatrix( Mix (Vector3(-10.0, 5.3, -9.0) , Vector3(0.0, 0.0, 0.0), per ));
					mat = mat * CreateEulerAnglesMatrix( Mix (Vector3(-1.5, 0.2, -1.8) , Vector3(0.0, 0.0, 0.0), per ));
				}
			}
		//reload animation of dry weapon (pump)
			if (reloadtype == 0)
			{
				if (reload < 0.52f)
				{
					//wait
				}
				else if (reload < 0.7f)
				{
					//moving
					float per = (reload - 0.52f)/0.18f;
					per = quadraticInOut(per);
					mat = mat * CreateTranslateMatrix( Mix (Vector3(0.0, 0.0, 0.0) , Vector3(0.0, 0.0, 0.0), per ));
					mat = mat * CreateEulerAnglesMatrix( Mix (Vector3(0.0, 0.0, 0.0) , Vector3(0.0, 0.0, 0.0), per ));
				}
				else if (reload < 0.75f)
				{
					float per = (reload - 0.7f)/0.05f;
					per = quadraticInOut(per);
					mat = mat * CreateTranslateMatrix( Mix (Vector3(0.0, 0.0, 0.0) , Vector3(0.0, 0.0, 0.0), per ));
					mat = mat * CreateEulerAnglesMatrix( Mix (Vector3(0.0, 0.0, 0.0) , Vector3(0.0, 0.0, 0.0), per ));
				}
				else if (reload < 1.0f)
				{
					float per = (reload - 0.75f)/0.25f;
					per = quadraticInOut(per);
					mat = mat * CreateTranslateMatrix( Mix (Vector3(0.0, 0.0, 0.0) , Vector3(0.0, 0.0, 0.0), per ));
					mat = mat * CreateEulerAnglesMatrix( Mix (Vector3(0.0, 0.0, 0.0) , Vector3(0.0, 0.0, 0.0), per ));
				}
			}

			if (!reloading && reload > 1.f && bullet > 0)
			{
				if (ReadyState < 0.1f)
				{
					//wait
				}
				else if (ReadyState < 0.5f)
				{
					float per = (ReadyState - 0.1f)/0.4f;
					per=quadraticInOut(per);
					mat *= CreateEulerAnglesMatrix(Mix(Vector3(0.0f,0.0f,0.0f),Vector3(0.1f,0.08f,0.05f),per));
					
				}
				else if (ReadyState < 0.7f)
				{
					float per = (ReadyState - 0.5f)/0.25f;
					per=quadraticInOut(per);
					mat *= CreateEulerAnglesMatrix(Mix(Vector3(0.1f,0.08f,0.05f),Vector3(0.15f,0.1f,0.07f),per));
					
				}
				else if (ReadyState < 0.75f)
				{
					float per = (ReadyState - 0.7f)/0.05f;
					per=quadraticInOut(per);
		
					mat *= CreateEulerAnglesMatrix(Mix(Vector3(0.15f,0.1f,0.07f),Vector3(0.15f,0.1f,0.07f),per));

					
				}
				else if (ReadyState < 1.f)
				{
					float per = (ReadyState - 0.75f)/0.25f;
					per=quadraticInOut(per);
					mat *= CreateEulerAnglesMatrix(Mix(Vector3(0.15f,0.1f,0.07f),Vector3(0.0f,0.0f,0.0f),per));
				}
			}


			//(x,y,z) (x+left/-right, y+further/-closer, z+down/-up)
			ModelRenderParam param;
			Matrix4 weapMatrix = eyeMatrix * mat;
			param.matrix = weapMatrix * CreateScaleMatrix(0.5f);
			param.depthHack = true;
			renderer.AddModel(gunModel, param);
			
			/*if (ReadyState < 0.5)
			{
				param.matrix = weapMatrix * CreateScaleMatrix(0.5f);
				param.depthHack = true;
				renderer.AddModel(trueModel, param);
			}*/
			
			// draw sights
			Matrix4 sightMat = weapMatrix;
			sightMat *= CreateTranslateMatrix(0.0f, 8.0f, -10.89f);
			sightMat *= CreateScaleMatrix(0.26f);
			param.matrix = sightMat;
			renderer.AddModel(sightModel2, param); // rear
			
			
			// reload action
			leftHand = mat * Vector3(0.f, 9.f, 2.f);
			rightHand = mat * Vector3(0.f, -20.f, 2.f);			
			
			//leftHand animation - inserting shells
			//good for now
			if(reloading) 
			{
				if(reload < 0.2f) //grab new bullet
				{
					float per = reload / 0.2f;
					per=quadraticInOut(per);
					leftHand = Mix(leftHand, leftHand2,per);
						
				}
				else if(reload < 0.35f) //insert bullet
				{
					float per = (reload - 0.2f) / 0.15f;
					per=quadraticInOut(per);
					leftHand = Mix(leftHand2, leftHand3,per);
				}
				else if(reload < 0.5f) //move hand to original
				{
					float per = (reload - 0.35f) / 0.15f;
					per=quadraticInOut(per);
					leftHand = Mix(leftHand3, leftHand,per);
				}
			}
			
			//BASIC PUMP ANIMATION
			param.matrix = weapMatrix * CreateScaleMatrix(0.5f);
			param.depthHack = true;	
			if (reloadtype == 0 && reload < 1.0f)
			{
				if (reload < 0.55f)
				{
					//wait
				}
				else if (reload < 0.60f)
				{
					float per = (reload - 0.55f) / 0.05f;
					per=quadraticInOut(per);
					param.matrix *= CreateTranslateMatrix(Mix (Vector3(0.0f, 0.0f, 0.0f), Vector3(0.f, 0.f, 0.f), per));
					leftHand = leftHand;
				}
				else if (reload < 0.75f)
				{
					float per = (reload -0.60f) / 0.15f;
					per=quadraticInOut(per);
					param.matrix *= CreateTranslateMatrix(Mix(Vector3(0.f, 0.f, 0.f),Vector3 (0.f, -7.f, 0.f), per));
					leftHand = Mix(leftHand,leftHand4,per);
				}
				else if (reload < 0.9f)
				{
					float per = (reload - 0.75f) / 0.15f;
					per=quadraticInOut(per);
					param.matrix *= CreateTranslateMatrix(Mix (Vector3(0.f, -7.f, 0.f), Vector3 (0.f, 0.f, 0.f),per));
					leftHand = Mix(leftHand4,leftHando,per);
				}
				else if (bullet == ClipSize)
				{
					reloadtype = 1;
				}
			}			
				//.10 - .50 move
				//.50 - .70 pump
				//.50 - .75 move down, stop
				//.75 - 1.0 pump & move orig
			
			else if (!reloading && reload > 1.5f && bullet > 0)
			{
				if (ReadyState < 0.55f)
				{
					//wait
				}
				else if (ReadyState < 0.60f)
				{
					float per = (ReadyState - 0.55f) / 0.05f;
					per=quadraticInOut(per);
					param.matrix *= CreateTranslateMatrix(Mix (Vector3(0.f, 0.f, 0.f), Vector3 (0.f, 0.f, 0.f),per));
					leftHand = leftHand;
				}
				else if (ReadyState < 0.75f)
				{
				    float per = (ReadyState -0.60f) / 0.15f;
					per=quadraticInOut(per);
					param.matrix *= CreateTranslateMatrix(Mix (Vector3(0.f, 0.f, 0.f), Vector3 (0.f, -7.f, 0.f),per));
					leftHand = Mix(leftHand,leftHand4,per);
				}
				else if (ReadyState < 0.9f)
				{
					float per = (ReadyState - 0.75f) / 0.15f;
					per=quadraticInOut(per);
					param.matrix *= CreateTranslateMatrix(Mix (Vector3(0.f, -7.f, 0.f), Vector3 (0.f, 0.f, 0.f),per));
					leftHand = Mix(leftHand4,leftHando,per);
				}
			}	
			renderer.AddModel(pumpModel, param);
			//(x,y,z) (x+left/-right, y+further/-closer, z+down/-up)
			
			LeftHandPosition = leftHand;
			RightHandPosition = rightHand;
		}
	}
	
	IWeaponSkin@ CreateViewShotgunSkin(Renderer@ r, AudioDevice@ dev) 
	{
		return ViewShotgunSkin(r, dev);
	}
}